-- Localization
local L = setmetatable( 
{
	["Update"] = "Update",
	["Next"] = "Next",
	["Test"] = "Test",
	["TriviaBotQuestionMaker"] = "TriviaBot QuizMaker",
	["Title"] = "Quiz List:",
	["NewTitle"] = "New Quiz",
	["Description"] = "Description:",
	["NewDescription"] = "Quiz Description",
	["Author"] = "Author:",
	["NewAuthor"] = "Quiz Author",
	["Category"] = "Category List:",
	["NewCategory"] = "Add Category",
	["Question"] = "Question:",
	["Answers"] = "Answers:",
	["Points"] = "Points:",
	["Browse"] = "Browse:",
	["Hints"] = "Hints:",
	["SaveSet"] = "Save List",
	["TriviaBotQuestionMakerNext"] = "Add or Update Question",
	["TriviaBot_QuestionMakerTitleInput"] = "Type a name for the Quiz and press Enter. It must be unique. Typing an existing name will select it.",
	["TriviaBot_QuestionMakerDescInput"] = "Type a description for the Quiz and press Enter. Typing a new one overwrites / corrects the previous.",
	["TriviaBot_QuestionMakerAuthInput"] = "Type an author name and press Enter. For example your player name or nickname.",
	["TriviaBot_QuestionMakerCatInput"] = "Type a category name and press Enter. Keep typing categories and pressing Enter to add more. Pressing Enter twice will move to the Question Fields.",
	["TriviaBot_QuestionMakerQuestionInput"] = "Type a question and press Enter.",
	["TriviaBot_QuestionMakerAnswersInput"] = "Type an answer or answers and press Enter. You can enter multiple answers by using ';' (Example: 2;two;second)",
	["TriviaBot_QuestionMakerPointsInput"] = "Type a number and press Enter. If left empty it will auto-fill with 1.",
	["TriviaBot_QuestionMakerNumInput"] = "Type a number or use the arrows to browse the existing questions.",
	["TriviaBot_QuestionMakerHintsInput"] = "Type a hint or hints and press Enter. You can enter multiple hints by using ';' (Example: abbreviation;acronym)",
	["TriviaBot_QuestionMakerNextBtn"] = "Press to save a new question. You can set a keybind from Game Options for mouse-free data entry.",
	["TriviaBot_QuestionMakerNextBtnU"] = "Press to update/correct the selected question. You can set a keybind from Game Options for mouse-free data entry.",
	["TriviaBot_QuestionMakerSaveBtn"] = "Press to ReloadUI and save the working quizzes. You can continue updating them in a future game session. The Quiz must contain at least 10 questions for the Button to enable.",
	["TriviaBot_QuestionMakerTriviaBotBtn"] = "Send the selected Quiz to TriviaBot for testing. You need to select a Quiz from the dropdown for the Button to enable.",
}, 
{
	__index = function ( self, Key )
		if ( Key ~= nil ) then
			rawset( self, Key, Key );
			return Key;
		end
	end;
} );
--[[ -- example for German
if ( GetLocale() == "deDE" ) then
	L = setmetatable( 
	{
		MAIN = "Main",
		ALTS = "Alts",
	}, 
	{ __index = L; } );
end
]]


-- event frame
local f=CreateFrame("Frame")
f:SetScript("OnEvent", 
function(self,event,...) 
	if self[event] then
		return self[event](...)
	end
end)
f:RegisterEvent("ADDON_LOADED")

-- locals
local TB_Question_Sets = {}; -- working table for sets
local TB_Questions = {}; -- Pointer to the active question set
local TB_CurrentSetID, TB_CurrentQuestionID, TB_NewQuestionID, TB_CurrentCatID
local TB_Answers = {}
local TB_Hints = {}
local TBQM_NONE = "n/a";
local NOP = function() end
local MIN_LIST, MAX_LIST = 9, 1000
local LIST_DELIM = ";"
BINDING_HEADER_TRIVIABOTQM = L["TriviaBotQuestionMaker"]
BINDING_NAME_TRIVIABOTQM_NEXT = L["TriviaBotQuestionMakerNext"]

-- utility functions
local function deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, _copy(getmetatable(object)))
    end
    return _copy(object)
end

local function findSide(frame)
	local side = "left";
	local rightDist = 0;
	local leftPos = frame:GetLeft();
	local rightPos = frame:GetRight();
	if ( not rightPos ) then
		rightPos = 0;
	end
	if ( not leftPos ) then
		leftPos = 0;
	end
	rightDist = GetScreenWidth() - rightPos;
	if (leftPos and (rightDist < leftPos)) then
		side = "left";
	else
		side = "right";
	end
	return side
end

local function sanitizedListToTable(...)
	local t = {}
	local text
	for i=1,select("#",...) do
		text = strtrim((select(i,...)))
		if text and text ~= "" then
			tinsert(t,text)
		end
	end
	return t
end

local function setValuesBySetID()
	if TB_CurrentSetID and TB_CurrentSetID > 0 then
		if TB_Question_Sets[TB_CurrentSetID] and TB_Question_Sets[TB_CurrentSetID]['Description'] then
			UIDropDownMenu_SetSelectedValue(f.gui.setDescription, TB_CurrentSetID)
			UIDropDownMenu_SetText(f.gui.setDescription,TB_Question_Sets[TB_CurrentSetID]['Description'])
		else
			UIDropDownMenu_SetSelectedValue(f.gui.setDescription,0)
		end
		if TB_Question_Sets[TB_CurrentSetID] and TB_Question_Sets[TB_CurrentSetID]['Author'] then
			UIDropDownMenu_SetSelectedValue(f.gui.setAuthor, TB_CurrentSetID)
			UIDropDownMenu_SetText(f.gui.setAuthor, TB_Question_Sets[TB_CurrentSetID]['Author'])
		else
			UIDropDownMenu_SetSelectedValue(f.gui.setAuthor,0)
		end
		if TB_Question_Sets[TB_CurrentSetID] and TB_Question_Sets[TB_CurrentSetID]['Categories'] then
			UIDropDownMenu_SetSelectedValue(f.gui.questionCategory,1)
			UIDropDownMenu_SetText(f.gui.questionCategory,TB_Question_Sets[TB_CurrentSetID]['Categories'][1])
			TB_CurrentCatID = 1
		else
			UIDropDownMenu_SetSelectedValue(f.gui.questionCategory,0)
		end
		if TB_Question_Sets[TB_CurrentSetID] and TB_Question_Sets[TB_CurrentSetID]['Question'] then
			TB_CurrentQuestionID = #(TB_Question_Sets[TB_CurrentSetID]['Question'])
			TB_NewQuestionID = TB_CurrentQuestionID + 1
			f.gui.questionRecord:SetText("["..TB_NewQuestionID.."]")
			if f.gui.testSetbtn then
				if TB_CurrentQuestionID > 0 then
					f.gui.testSetbtn:Enable()
				else
					f.gui.testSetbtn:Disable()
				end
			end
			if TB_NewQuestionID > MAX_LIST then
				f.gui.nextRecordbtn:Disable()
			else
				f.gui.nextRecordbtn:Enable()
			end
		else
			f.gui.questionRecord:SetText("[#]")
			if f.gui.testSetbtn then
				f.gui.testSetbtn:Disable()
			end
		end
		f.gui.setRecord:SetText(TB_CurrentSetID and "["..TB_CurrentSetID.."]")
	else
		f.gui.setRecord:SetText("[#]")
	end
end

-- gui functions
local function setControlTooltip(control)
	local controlname = control:GetName()
	if control.updateMode then controlname = controlname.."U" end
	GameTooltip:SetOwner(control,"ANCHOR_TOP")
	GameTooltip:SetWidth(200)
	GameTooltip:AddLine(L[controlname],NORMAL_FONT_COLOR.r,NORMAL_FONT_COLOR.g,NORMAL_FONT_COLOR.b,true)
	GameTooltip:Show();
end

local function setTitleDropdown_OnClick(self)
	TB_CurrentSetID = (self.value > 0) and self.value or nil
	UIDropDownMenu_SetSelectedValue(f.gui.setTitle, self.value)
	setValuesBySetID()
end

local function setCatDropdown_OnClick(self)
	TB_CurrentCatID = (self.value) > 0 and self.value or nil
	UIDropDownMenu_SetSelectedValue(f.gui.questionCategory, self.value)
end

local function setTitleDropdown_Initialize()
	local info = UIDropDownMenu_CreateInfo()
 	info.value = 0
	info.text = TBQM_NONE
 	info.func = setTitleDropdown_OnClick
 	UIDropDownMenu_AddButton(info)
 	UIDropDownMenu_SetSelectedValue(TriviaBot_QuestionMakerTitleDrop,0)
	for i, set in pairs(TB_Question_Sets) do
 		info = UIDropDownMenu_CreateInfo();
		info.value = i
		info.text = set['Title']
		info.func = setTitleDropdown_OnClick
		UIDropDownMenu_AddButton(info)
	end
end
local function setDescDropdown_Initialize()
	local info = UIDropDownMenu_CreateInfo()
	info.value = 0
	info.text = TBQM_NONE
 	info.func = NOP
	UIDropDownMenu_AddButton(info)
	UIDropDownMenu_SetSelectedValue(TriviaBot_QuestionMakerDescDrop,0)
	for i, set in pairs(TB_Question_Sets) do
 		info = UIDropDownMenu_CreateInfo();
		info.value = i
		info.text = set['Description']
 		info.func = NOP
		UIDropDownMenu_AddButton(info)
	end
	TriviaBot_QuestionMakerDescDropButton:Disable()
end
local function setAuthDropdown_Initialize()
	local info = UIDropDownMenu_CreateInfo()
	info.value = 0
	info.text = TBQM_NONE
 	info.func = NOP
	UIDropDownMenu_AddButton(info)
	UIDropDownMenu_SetSelectedValue(TriviaBot_QuestionMakerAuthDrop,0)
	for i, set in pairs(TB_Question_Sets) do
 		info = UIDropDownMenu_CreateInfo();
		info.value = i
		info.text = set['Author']
 		info.func = NOP
		UIDropDownMenu_AddButton(info)
	end
	TriviaBot_QuestionMakerAuthDropButton:Disable()
end
local function setCatDropdown_Initialize()
	local info = UIDropDownMenu_CreateInfo()
	info.value = 0
	info.text = TBQM_NONE
	info.func = setCatDropdown_OnClick
	UIDropDownMenu_AddButton(info)
	UIDropDownMenu_SetSelectedValue(TriviaBot_QuestionMakerCatDrop,0)
	if not TB_CurrentSetID then return end
	TB_Questions = TB_Question_Sets[TB_CurrentSetID]
	if TB_Questions['Categories'] then
		for i, category in pairs(TB_Questions['Categories']) do
 			info = UIDropDownMenu_CreateInfo();
			info.value = i
			info.text = category
			info.func = setCatDropdown_OnClick
			UIDropDownMenu_AddButton(info)
		end
	end
end

local function setTitleInput_OnEnterPressed(self)
	local text = self:GetText()
	if text then text = strtrim(text) end
	if text ~= "" then
		local found
		for i, set in pairs(TB_Question_Sets) do
			if strlower(text) == strlower(set['Title']) then
				found = true
				UIDropDownMenu_SetSelectedValue(f.gui.setTitle, i)
				TB_CurrentSetID = i
				setValuesBySetID()
				break
			end
		end
		if not found then
 			tinsert(TB_Question_Sets, {['Title']=text})
			TB_CurrentSetID = #TB_Question_Sets
			UIDropDownMenu_ClearAll(f.gui.setTitle)
			UIDropDownMenu_Initialize(f.gui.setTitle,setTitleDropdown_Initialize)
			UIDropDownMenu_SetSelectedValue(f.gui.setTitle, TB_CurrentSetID)
			setValuesBySetID()
		end
	end
	self:SetText("")
	self:ClearFocus()
	f.gui.setDescEdit:SetFocus()
end
local function setTitleInput_OnEscapePressed(self)
	self:ClearFocus()
end

local function setDescInput_OnEnterPressed(self)
	local text = self:GetText()
	if text then text = strtrim(text) end
	if text ~= "" then
		if TB_CurrentSetID then
			TB_Question_Sets[TB_CurrentSetID]['Description'] = text -- set it or update it
			UIDropDownMenu_ClearAll(f.gui.setDescription)
			UIDropDownMenu_Initialize(f.gui.setDescription,setDescDropdown_Initialize)
			UIDropDownMenu_SetSelectedValue(f.gui.setDescription, TB_CurrentSetID)
		else
			print("You need to select a List.")
		end
	end
	self:SetText("")
	self:ClearFocus()
	f.gui.setAuthorEdit:SetFocus()
end
local function setDescInput_OnEscapePressed(self)
	self:ClearFocus()
end

local function setAuthInput_OnEnterPressed(self)
	local text = self:GetText()
	if text then text = strtrim(text) end
	if text ~= "" then
		if TB_CurrentSetID then
			TB_Question_Sets[TB_CurrentSetID]['Author'] = text -- set it or update it
			UIDropDownMenu_ClearAll(f.gui.setAuthor)
			UIDropDownMenu_Initialize(f.gui.setAuthor,setAuthDropdown_Initialize)
			UIDropDownMenu_SetSelectedValue(f.gui.setAuthor, TB_CurrentSetID)
		else
			print("You need to select a List.")
		end
	end
	self:SetText("")
	self:ClearFocus()
	f.gui.questionCatEdit:SetFocus()
end
local function setAuthInput_OnEscapePressed(self)
	self:ClearFocus()
end

local function setCatInput_OnEnterPressed(self)
	local text = self:GetText()
	if text then text = strtrim(text) end
	if text ~= "" then
		if TB_CurrentSetID then
			TB_Questions = TB_Question_Sets[TB_CurrentSetID]
			if not TB_Questions['Categories'] then
				TB_Questions['Categories'] = {}
				tinsert(TB_Questions['Categories'],text)
				TB_CurrentCatID = #(TB_Questions['Categories'])
				UIDropDownMenu_ClearAll(f.gui.questionCategory)
				UIDropDownMenu_Initialize(f.gui.questionCategory,setCatDropdown_Initialize)
				UIDropDownMenu_SetSelectedValue(f.gui.questionCategory, TB_CurrentCatID)
			else
				local found
				for i, cat in pairs(TB_Questions['Categories']) do
					if strlower(text) == strlower(cat) then
						found = true
						UIDropDownMenu_SetSelectedValue(f.gui.questionCategory,i)
						TB_CurrentCatID = i
						break
					end
				end
				if not found then
					tinsert(TB_Questions['Categories'],text)
					TB_CurrentCatID = #(TB_Questions['Categories'])
					UIDropDownMenu_ClearAll(f.gui.questionCategory)
					UIDropDownMenu_Initialize(f.gui.questionCategory,setCatDropdown_Initialize)
					UIDropDownMenu_SetSelectedValue(f.gui.questionCategory, TB_CurrentCatID)
				end
			end
		else
			print("You need to select a List.")
		end
	else
		self:ClearFocus()
		f.gui.questionEdit:SetFocus()
	end
	self:SetText("")
end
local function setCatInput_OnEscapePressed(self)
	self:ClearFocus()
end

local function setQuestionInput_OnEnterPressed(self)
	self:ClearFocus()
	f.gui.answerEdit:SetFocus()
end
local function setQuestionInput_OnEscapePressed(self)
	self:ClearFocus()
end

local function setAnswersInput_OnEnterPressed(self)
	self:ClearFocus()
	f.gui.pointsEdit:SetFocus()
end
local function setAnswersInput_OnEscapePressed(self)
	self:ClearFocus()
end

local function setPointsInput_OnEnterPressed(self)
	local pointText = self:GetText()
	if pointText and pointText == "" then
		self:SetText("1")
	end
	self:ClearFocus()
	f.gui.hintsEdit:SetFocus()
end
local function setPointsInput_OnEscapePressed(self)
	self:ClearFocus()
end

local function setHintsInput_OnEnterPressed(self)
	self:ClearFocus()
end
local function setHintsInput_OnEscapePressed(self)
	self:ClearFocus()
end

local function questionNumInput_OnTextChanged(self)
	local numtext, num = self:GetText()
	num = tonumber(numtext)
	if ( num and num <= 0 ) then
		self:SetText("1");
		num = 1
	end
	if num then
		if TB_CurrentSetID and TB_Question_Sets[TB_CurrentSetID] and TB_Question_Sets[TB_CurrentSetID]['Question'] then
			local records = #(TB_Question_Sets[TB_CurrentSetID]['Question'])
			if records > 0 then
				if num >= records then
					f.gui.nextQbtn:Disable()
					f.gui.prevQbtn:Enable()
					num = records
					self:SetText(tostring(num))
				elseif num == 1 then
					f.gui.prevQbtn:Disable()
					if records > 1 then
						f.gui.nextQbtn:Enable()
					else
						f.gui.nextQbtn:Disable()
					end
				elseif num > 1 and num < records then
					f.gui.prevQbtn:Enable()
					f.gui.nextQbtn:Enable()
				end
				TB_CurrentQuestionID = num
				TB_CurrentCatID = TB_Question_Sets[TB_CurrentSetID]['Category'][TB_CurrentQuestionID]
				UIDropDownMenu_SetSelectedValue(f.gui.questionCategory, TB_CurrentCatID)
				UIDropDownMenu_SetText(f.gui.questionCategory,TB_Question_Sets[TB_CurrentSetID]['Categories'][TB_CurrentCatID])
				f.gui.questionEdit:SetText(TB_Question_Sets[TB_CurrentSetID]['Question'][TB_CurrentQuestionID])
				f.gui.answerEdit:SetText(strjoin(LIST_DELIM,unpack(TB_Question_Sets[TB_CurrentSetID]['Answers'][TB_CurrentQuestionID])))
				f.gui.pointsEdit:SetText(TB_Question_Sets[TB_CurrentSetID]['Points'][TB_CurrentQuestionID])
				if next(TB_Question_Sets[TB_CurrentSetID]['Hints'][TB_CurrentQuestionID]) then
					f.gui.hintsEdit:SetText(strjoin(LIST_DELIM,unpack(TB_Question_Sets[TB_CurrentSetID]['Hints'][TB_CurrentQuestionID])))
				else
					f.gui.hintsEdit:SetText("")
				end
				f.gui.nextRecordbtn.updateMode = true
				f.gui.nextRecordbtn:Enable()
				f.gui.nextRecordbtn:SetText(L["Update"])
			end
		else
			self:SetText("") -- no set loaded or no questions in set yet
		end
	end
end
local function questionNumInput_OnEnterPressed(self)
	questionNumInput_OnTextChanged(self)
	self:ClearFocus()
end
local function questionNumInput_OnEscapePressed(self)
	self:ClearFocus()
end
local function questionNumInput_OnEditFocusLost(self)
	EditBox_ClearHighlight(self)
end
local function questionNumInput_OnEditFocusGained(self)
	EditBox_HighlightText(self)
end

local function prevQuestionbtn_OnClick(self)
	local num = tonumber(f.gui.browseEdit:GetText())
	if not num or num < 3 then
		f.gui.browseEdit:SetText("1")
	else
		local numtext = tostring(num-1)
		f.gui.browseEdit:SetText(numtext)
	end
end
local function nextQuestionbtn_OnClick(self)
	local num = tonumber(f.gui.browseEdit:GetText())
	if not num then
		f.gui.browseEdit:SetText("1")
	else
		local numtext = tostring(num+1)
		f.gui.browseEdit:SetText(numtext)
	end
end

local function nextbtn_OnClick(self)
	local questionText = f.gui.questionEdit:GetText()
	if questionText then questionText = strtrim(questionText) end
	local answersText = f.gui.answerEdit:GetText()
	if answersText then answersText = strtrim(answersText) end
	local pointsText = f.gui.pointsEdit:GetText()
	if pointsText and pointsText == "" then pointsText = "1" end
	local hintsText = f.gui.hintsEdit:GetText()
	if hintsText then hintsText = strtrim(hintsText) end
	if not questionText or questionText=="" then
		print("Missing or invalid question text")
		f.gui.questionEdit:SetFocus()
		return
	end
	if not answersText or answersText=="" then
		print("Missing or invalid question text")
		f.gui.answerEdit:SetFocus()
		return
	end
	TB_Answers = sanitizedListToTable(strsplit(LIST_DELIM,answersText))
	if not next(TB_Answers) then
		print("Missing or invalid question text")
		f.gui.answerEdit:SetFocus()
		return
	end
	if hintsText == "" then
		TB_Hints = {}
	else
		TB_Hints = sanitizedListToTable(strsplit(LIST_DELIM,hintsText))
	end
	if TB_CurrentSetID
	and TB_Question_Sets[TB_CurrentSetID]
	and TB_Question_Sets[TB_CurrentSetID]['Title']
	and TB_Question_Sets[TB_CurrentSetID]['Description']
	and TB_Question_Sets[TB_CurrentSetID]['Author']
	and TB_Question_Sets[TB_CurrentSetID]['Categories']
	and TB_Question_Sets[TB_CurrentSetID]['Categories'][TB_CurrentCatID] then
		if not TB_Question_Sets[TB_CurrentSetID]['Question'] then TB_Question_Sets[TB_CurrentSetID]['Question'] = {} end
		if not TB_Question_Sets[TB_CurrentSetID]['Answers'] then TB_Question_Sets[TB_CurrentSetID]['Answers'] = {} end
		if not TB_Question_Sets[TB_CurrentSetID]['Category'] then TB_Question_Sets[TB_CurrentSetID]['Category'] = {} end
		if not TB_Question_Sets[TB_CurrentSetID]['Points'] then TB_Question_Sets[TB_CurrentSetID]['Points'] = {} end
		if not TB_Question_Sets[TB_CurrentSetID]['Hints'] then TB_Question_Sets[TB_CurrentSetID]['Hints'] = {} end
		if self.updateMode then
			TB_Question_Sets[TB_CurrentSetID]['Question'][TB_CurrentQuestionID] = questionText
			TB_Question_Sets[TB_CurrentSetID]['Answers'][TB_CurrentQuestionID] = TB_Answers
			TB_Question_Sets[TB_CurrentSetID]['Category'][TB_CurrentQuestionID] = TB_CurrentCatID
			TB_Question_Sets[TB_CurrentSetID]['Points'][TB_CurrentQuestionID] = tonumber(pointsText)
			TB_Question_Sets[TB_CurrentSetID]['Hints'][TB_CurrentQuestionID] = TB_Hints
			self.updateMode = nil
			self:SetText(L["Next"])
		else
			tinsert(TB_Question_Sets[TB_CurrentSetID]['Question'],questionText)
			tinsert(TB_Question_Sets[TB_CurrentSetID]['Answers'],TB_Answers)
			tinsert(TB_Question_Sets[TB_CurrentSetID]['Category'],TB_CurrentCatID)
			tinsert(TB_Question_Sets[TB_CurrentSetID]['Points'],tonumber(pointsText))
			tinsert(TB_Question_Sets[TB_CurrentSetID]['Hints'],TB_Hints)
		end
		TB_CurrentQuestionID = #(TB_Question_Sets[TB_CurrentSetID]['Question'])
		TB_NewQuestionID = TB_CurrentQuestionID + 1
		f.gui.questionRecord:SetText("["..TB_NewQuestionID.."]")
		f.gui.answerEdit:SetText("")
		f.gui.hintsEdit:SetText("")
		f.gui.questionEdit:SetText("")
		f.gui.questionEdit:SetFocus() -- and move to next question
		if TB_NewQuestionID > MIN_LIST then f.gui.reloadbtn:Enable() end
		if TB_NewQuestionID > MAX_LIST then 
			self:Disable(); 
			print("You have reached the maximum number of questions for a list. Please save it and start a new one.") 
		else
			self:Enable();
		end
		f.gui.nextQbtn:Enable();
	else -- missing mandatory fields, title,description,author etc
		print("Missing mandatory info (List, Description, Author or Category).")
	end
end

local function testbtn_OnClick(self)
	if TB_CurrentSetID then
		wipe(_G["TriviaBotQuestionMakerExports"]["ActiveSet"])
		_G["TriviaBotQuestionMakerExports"]["ActiveSet"][1] = deepcopy(TB_Question_Sets[TB_CurrentSetID])
		TriviaBot_LoadQuestionMakerSet()
		TriviaBotGUI_Update()
		if not TriviaBotGUI_Header:IsShown() then
 			TriviaBotGUI_Header:ClearAllPoints()
			local side = findSide(f.gui)
			f.gui:ClearAllPoints()
			if side == "left" then
				TriviaBotGUI_Header:SetPoint("TOPRIGHT",f.gui,"TOPLEFT",0,32)
			else
				TriviaBotGUI_Header:SetPoint("TOPLEFT",f.gui,"TOPRIGHT",0,32)
			end
			TriviaBotGUI_Header:Show()
		end
	end
end

local function savebtn_OnClick(self)
	-- check min (50), max (1000) questions
	-- and reloadui
	TriviaBot_QuestionsQM = TB_Question_Sets
	ReloadUI()
end

-- main frame
local function createGUI()
	local gui=CreateFrame("Frame","TriviaBot_QuestionMakerGUI",UIParent)
	tinsert(UISpecialFrames,"TriviaBot_QuestionMakerGUI")
	gui:SetClampedToScreen(true)
	gui:SetWidth(500);
	gui:SetHeight(445);
	gui:SetBackdrop(
	  {
	     bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	     edgeFile = "Interface/Tooltips/UI-Tooltip-Border", -- "Interface/DialogFrame/UI-DialogBox-Border", 
	     tile = true, tileSize = 32, edgeSize = 16,
	     insets = {left = 5, right = 5, top = 5, bottom = 5}
	  });
	gui:SetBackdropBorderColor(0.8, 0.8, 0.8, 1);
	gui:SetBackdropColor(0.15, 0.15, 0.15, 1);
	gui:SetFrameStrata("BACKGROUND");
	gui:SetMovable(true);
	gui:EnableMouse(true);
	gui:SetPoint("CENTER");
	local title = gui:CreateTitleRegion()
	title:SetPoint("TOP")
	title:SetWidth(gui:GetWidth());
	title:SetHeight(40)
		
	-- Header Texture
	local header = gui:CreateTexture(nil, "ARTWORK");
	header:SetWidth(350);
	header:SetHeight(64);
	header:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header");
	header:SetPoint("TOP", gui, "TOP", 0, 12);
	-- Header Label
	local headerlabel = gui:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	headerlabel:SetPoint("TOP", header, "TOP", 0, -14);
	headerlabel:SetText(L["TriviaBotQuestionMaker"])
	
	-- Close Button
	local closebtn = CreateFrame("Button", nil, gui, "UIPanelCloseButton");
	closebtn:SetPoint("TOPRIGHT");
	closebtn:SetScript("OnClick", function(self,button) gui:Hide(); end);
	
	-- Question Set
	local questionSetSection = CreateFrame("Frame",nil,gui)
	questionSetSection:SetWidth(480);
	questionSetSection:SetHeight(150);
	questionSetSection:SetPoint("TOPLEFT", 10, -25)
	questionSetSection:SetBackdrop(
		{
	     bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	     edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
	     tile = true, tileSize = 8, edgeSize = 8,
	     insets = {left = 2, right = 2, top = 2, bottom = 2}
	  });
	questionSetSection:SetBackdropBorderColor(0, 0, 0, 1);
	questionSetSection:SetBackdropColor(0.1, 0.1, 0.1, 1);	
	-- Question Set Title
	local setTitleDropdown = CreateFrame("Frame", "TriviaBot_QuestionMakerTitleDrop", questionSetSection, "UIDropDownMenuTemplate");
	gui.setTitle = setTitleDropdown
	UIDropDownMenu_SetWidth(setTitleDropdown, 180);
	UIDropDownMenu_SetButtonWidth(setTitleDropdown, 20);
	setTitleDropdown:SetPoint("TOPLEFT", questionSetSection, "TOPLEFT", 5, -20);
	UIDropDownMenu_Initialize(setTitleDropdown, setTitleDropdown_Initialize);
	-- Question Set Title label
	local setTitleDropdownlabel = setTitleDropdown:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setTitleDropdownlabel:SetText(L["Title"]);
	setTitleDropdownlabel:SetPoint("BOTTOMLEFT", setTitleDropdown, "TOPLEFT", 20, 2);
	-- Set Record
	local setRecordlabel = setTitleDropdown:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	gui.setRecord = setRecordlabel
	setRecordlabel:SetText("[#]");
	setRecordlabel:SetTextColor(0.0,1.0,0.0,1);
	setRecordlabel:SetPoint("LEFT", setTitleDropdown, "RIGHT", -10, 4)
	-- New Set Title
	local setTitleInput = CreateFrame("EditBox", "TriviaBot_QuestionMakerTitleInput", questionSetSection, "InputBoxTemplate");
	gui.setTitleEdit = setTitleInput
	setTitleInput:SetWidth(210);
	setTitleInput:SetHeight(36);
	setTitleInput:SetMaxLetters(100);
	setTitleInput:SetAutoFocus(false);
	setTitleInput:SetPoint("LEFT", setTitleDropdown, "RIGHT", 25, 2);
	setTitleInput:SetScript("OnEnterPressed", setTitleInput_OnEnterPressed);
	setTitleInput:SetScript("OnEscapePressed", setTitleInput_OnEscapePressed);
	setTitleInput:SetScript("OnEnter",setControlTooltip);
	setTitleInput:SetScript("OnLeave",GameTooltip_Hide);

	-- New Set Title Label
	local setTitleInputlabel = setTitleInput:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setTitleInputlabel:SetText(L["NewTitle"]);
	setTitleInputlabel:SetPoint("BOTTOMLEFT", setTitleInput, "TOPLEFT", -4, -2);
	-- Question Set Description
	local setDescDropdown = CreateFrame("Frame", "TriviaBot_QuestionMakerDescDrop", questionSetSection, "UIDropDownMenuTemplate");
	gui.setDescription = setDescDropdown
	UIDropDownMenu_SetWidth(setDescDropdown, 200);
	UIDropDownMenu_SetButtonWidth(setDescDropdown, 20);
	setDescDropdown:SetPoint("TOPLEFT", setTitleDropdown, "BOTTOMLEFT", 0, -15);
	UIDropDownMenu_Initialize(setDescDropdown, setDescDropdown_Initialize);
	-- Question Set Description Label
	local setDescDropdownlabel = setDescDropdown:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setDescDropdownlabel:SetText(L["Description"]);
	setDescDropdownlabel:SetPoint("BOTTOMLEFT",setDescDropdown, "TOPLEFT", 20, 2);
	-- New Description
	local setDescInput = CreateFrame("EditBox", "TriviaBot_QuestionMakerDescInput", questionSetSection, "InputBoxTemplate");
	gui.setDescEdit = setDescInput
	setDescInput:SetWidth(210);
	setDescInput:SetHeight(36);
	setDescInput:SetMaxLetters(100);
	setDescInput:SetAutoFocus(false);
	setDescInput:SetPoint("LEFT", setDescDropdown, "RIGHT", 5, 2);
	setDescInput:SetScript("OnEnterPressed", setDescInput_OnEnterPressed);
	setDescInput:SetScript("OnEscapePressed", setDescInput_OnEscapePressed);
	setDescInput:SetScript("OnEnter",setControlTooltip);
	setDescInput:SetScript("OnLeave",GameTooltip_Hide);

	-- New Description Label
	local setDescInputlabel = setDescInput:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setDescInputlabel:SetText(L["NewDescription"]);
	setDescInputlabel:SetPoint("BOTTOMLEFT", setDescInput, "TOPLEFT", -4, -2);
	
		-- Question Set Author
	local setAuthDropdown = CreateFrame("Frame", "TriviaBot_QuestionMakerAuthDrop", questionSetSection, "UIDropDownMenuTemplate");
	gui.setAuthor = setAuthDropdown
	UIDropDownMenu_SetWidth(setAuthDropdown, 200);
	UIDropDownMenu_SetButtonWidth(setAuthDropdown, 20);
	setAuthDropdown:SetPoint("TOPLEFT", setDescDropdown, "BOTTOMLEFT", 0, -15);
	UIDropDownMenu_Initialize(setAuthDropdown, setAuthDropdown_Initialize);
	-- Question Set Author Label
	local setAuthDropdownlabel = setAuthDropdown:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setAuthDropdownlabel:SetText(L["Author"]);
	setAuthDropdownlabel:SetPoint("BOTTOMLEFT", setAuthDropdown, "TOPLEFT", 20, 2);
		-- New Author
	local setAuthInput = CreateFrame("EditBox", "TriviaBot_QuestionMakerAuthInput", questionSetSection, "InputBoxTemplate");
	gui.setAuthorEdit = setAuthInput
	setAuthInput:SetWidth(210);
	setAuthInput:SetHeight(36);
	setAuthInput:SetMaxLetters(100);
	setAuthInput:SetAutoFocus(false);
	setAuthInput:SetPoint("LEFT", setAuthDropdown, "RIGHT", 5, 2);
	setAuthInput:SetScript("OnEnterPressed", setAuthInput_OnEnterPressed);
	setAuthInput:SetScript("OnEscapePressed", setAuthInput_OnEscapePressed);
	setAuthInput:SetScript("OnEnter",setControlTooltip);
	setAuthInput:SetScript("OnLeave",GameTooltip_Hide);

	-- New Author Label
	local setAuthInputlabel = setAuthInput:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setAuthInputlabel:SetText(L["NewAuthor"]);
	setAuthInputlabel:SetPoint("BOTTOMLEFT", setAuthInput, "TOPLEFT", -4, -2);	
	
	-- Question
	local questionSection = CreateFrame("Frame",nil,gui)
	questionSection:SetWidth(480);
	questionSection:SetHeight(215);
	questionSection:SetPoint("TOPLEFT", questionSetSection, "BOTTOMLEFT", 0, -15)
	questionSection:SetBackdrop(
		{
	     bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	     edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
	     tile = true, tileSize = 8, edgeSize = 8,
	     insets = {left = 2, right = 2, top = 2, bottom = 2}
	  });
	questionSection:SetBackdropBorderColor(0, 0, 0, 1);
	questionSection:SetBackdropColor(0.1, 0.1, 0.1, 1);
	
	-- Category
	local setCatDropdown = CreateFrame("Frame", "TriviaBot_QuestionMakerCatDrop", questionSection, "UIDropDownMenuTemplate");
	gui.questionCategory = setCatDropdown
	UIDropDownMenu_SetWidth(setCatDropdown, 200);
	UIDropDownMenu_SetButtonWidth(setCatDropdown, 20);
	setCatDropdown:SetPoint("TOPLEFT", questionSection, "TOPLEFT", 5, -20);
	UIDropDownMenu_Initialize(setCatDropdown, setCatDropdown_Initialize);
	-- Category Label
	local setCatDropdownlabel = setCatDropdown:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setCatDropdownlabel:SetText(L["Category"]);
	setCatDropdownlabel:SetPoint("BOTTOMLEFT", setCatDropdown, "TOPLEFT", 20, 2);
	-- New Category
	local setCatInput = CreateFrame("EditBox", "TriviaBot_QuestionMakerCatInput", questionSection, "InputBoxTemplate");
	gui.questionCatEdit = setCatInput
	setCatInput:SetWidth(210);
	setCatInput:SetHeight(36);
	setCatInput:SetMaxLetters(100);
	setCatInput:SetAutoFocus(false);
	setCatInput:SetPoint("LEFT", setCatDropdown, "RIGHT", 5, 2);
	setCatInput:SetScript("OnEnterPressed", setCatInput_OnEnterPressed);
	setCatInput:SetScript("OnEscapePressed", setCatInput_OnEscapePressed);
	setCatInput:SetScript("OnEnter",setControlTooltip);
	setCatInput:SetScript("OnLeave",GameTooltip_Hide);

	-- New Category Label
	local setCatInputlabel = setCatInput:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setCatInputlabel:SetText(L["NewCategory"]);
	setCatInputlabel:SetPoint("BOTTOMLEFT", setCatInput, "TOPLEFT", -4, -2);
	
	-- question (maxletters 265)
	local setQuestionInput = CreateFrame("EditBox", "TriviaBot_QuestionMakerQuestionInput", questionSection, "InputBoxTemplate");
	gui.questionEdit = setQuestionInput
	setQuestionInput:SetWidth(415);
	setQuestionInput:SetHeight(36);
	setQuestionInput:SetMaxLetters(265);
	setQuestionInput:SetAutoFocus(false);
	setQuestionInput:SetPoint("TOPLEFT", setCatDropdown, "BOTTOMLEFT", 23, -10);
	setQuestionInput:SetScript("OnEnterPressed", setQuestionInput_OnEnterPressed);
	setQuestionInput:SetScript("OnEscapePressed", setQuestionInput_OnEscapePressed);
	setQuestionInput:SetScript("OnEnter",setControlTooltip);
	setQuestionInput:SetScript("OnLeave",GameTooltip_Hide);

	-- question label
	local setQuestionInputlabel = setQuestionInput:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setQuestionInputlabel:SetText(L["Question"]);
	setQuestionInputlabel:SetPoint("BOTTOMLEFT", setQuestionInput, "TOPLEFT", -4, -2);
	-- Question record label
	local setQuestionRecordlabel = setQuestionInput:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	gui.questionRecord = setQuestionRecordlabel
	setQuestionRecordlabel:SetText("[#]");
	setQuestionRecordlabel:SetTextColor(1.0,0.0,0.0,1);
	setQuestionRecordlabel:SetPoint("LEFT", setQuestionInput, "RIGHT", 5, 2)
	
	-- answers
	local setAnswersInput = CreateFrame("EditBox", "TriviaBot_QuestionMakerAnswersInput", questionSection, "InputBoxTemplate");
	gui.answerEdit = setAnswersInput
	setAnswersInput:SetWidth(370);
	setAnswersInput:SetHeight(36);
	setAnswersInput:SetMaxLetters(250);
	setAnswersInput:SetAutoFocus(false);
	setAnswersInput:SetPoint("TOPLEFT", setQuestionInput, "BOTTOMLEFT", 0, -10);
	setAnswersInput:SetScript("OnEnterPressed", setAnswersInput_OnEnterPressed);
	setAnswersInput:SetScript("OnEscapePressed", setAnswersInput_OnEscapePressed);
	setAnswersInput:SetScript("OnEnter",setControlTooltip);
	setAnswersInput:SetScript("OnLeave",GameTooltip_Hide);

	-- answers label
	local setAnswersInputlabel = setAnswersInput:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setAnswersInputlabel:SetText(L["Answers"]);
	setAnswersInputlabel:SetPoint("BOTTOMLEFT", setAnswersInput, "TOPLEFT", -4, -2);
	
	-- points
	local setPointsInput = CreateFrame("EditBox", "TriviaBot_QuestionMakerPointsInput", questionSection, "InputBoxTemplate");
	gui.pointsEdit = setPointsInput
	setPointsInput:SetWidth(50);
	setPointsInput:SetHeight(36);
	setPointsInput:SetNumeric(true);
	setPointsInput:SetMaxLetters(3);
	setPointsInput:SetAutoFocus(false);
	setPointsInput:SetPoint("LEFT", setAnswersInput, "RIGHT", 20, 0);
	setPointsInput:SetScript("OnEnterPressed", setPointsInput_OnEnterPressed);
	setPointsInput:SetScript("OnEscapePressed", setPointsInput_OnEscapePressed);
	setPointsInput:SetScript("OnEnter",setControlTooltip);
	setPointsInput:SetScript("OnLeave",GameTooltip_Hide);

	-- points label
	local setPointsInputlabel = setPointsInput:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setPointsInputlabel:SetText(L["Points"]);
	setPointsInputlabel:SetPoint("BOTTOMLEFT", setPointsInput, "TOPLEFT", -4, -2);
	
	-- browse questions
	local prevQuestionbtn = CreateFrame("Button", nil, questionSection);
	gui.prevQbtn = prevQuestionbtn
	prevQuestionbtn:SetWidth(26);
	prevQuestionbtn:SetHeight(25);
	prevQuestionbtn:SetPoint("TOPLEFT", setAnswersInput, "BOTTOMLEFT", -7, -10);
	prevQuestionbtn:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up");
	prevQuestionbtn:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down");
	prevQuestionbtn:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Disabled");
	prevQuestionbtn:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight", "ADD");
	prevQuestionbtn:SetScript("OnClick", prevQuestionbtn_OnClick);
	local questionBrowselabel = prevQuestionbtn:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
	questionBrowselabel:SetText(L["Browse"]);
	questionBrowselabel:SetPoint("BOTTOMLEFT", prevQuestionbtn, "TOPLEFT", 2, -2);
	
	local questionNumInput = CreateFrame("EditBox", "TriviaBot_QuestionMakerNumInput", questionSection, "InputBoxTemplate");
	gui.browseEdit = questionNumInput
	questionNumInput:SetWidth(40);
	questionNumInput:SetHeight(36);
	questionNumInput:SetNumeric(true);
	questionNumInput:SetMaxLetters(3);
	questionNumInput:SetAutoFocus(false);
	questionNumInput:SetPoint("LEFT", prevQuestionbtn, "RIGHT", 0, 0);
	questionNumInput:SetScript("OnEnterPressed", questionNumInput_OnEnterPressed);
	questionNumInput:SetScript("OnEscapePressed", questionNumInput_OnEscapePressed);
	questionNumInput:SetScript("OnEditFocusLost", questionNumInput_OnEditFocusLost);
	questionNumInput:SetScript("OnEditFocusGained", questionNumInput_OnEditFocusGained);
	questionNumInput:SetScript("OnTextChanged", questionNumInput_OnTextChanged);
	questionNumInput:SetScript("OnEnter",setControlTooltip);
	questionNumInput:SetScript("OnLeave",GameTooltip_Hide);
	
	
	local nextQuestionbtn = CreateFrame("Button", nil, questionSection);
	gui.nextQbtn = nextQuestionbtn
	nextQuestionbtn:SetWidth(26);
	nextQuestionbtn:SetHeight(25);
	nextQuestionbtn:SetPoint("LEFT", questionNumInput, "RIGHT", -3, 0);
	nextQuestionbtn:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up");
	nextQuestionbtn:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down");
	nextQuestionbtn:SetDisabledTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Disabled");
	nextQuestionbtn:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight", "ADD");
	nextQuestionbtn:SetScript("OnClick", nextQuestionbtn_OnClick);	
	
	-- hints
	local setHintsInput = CreateFrame("EditBox", "TriviaBot_QuestionMakerHintsInput", questionSection, "InputBoxTemplate");
	gui.hintsEdit = setHintsInput
	setHintsInput:SetWidth(347);
	setHintsInput:SetHeight(36);
	setHintsInput:SetMaxLetters(250);
	setHintsInput:SetAutoFocus(false);
	setHintsInput:SetPoint("LEFT", nextQuestionbtn, "RIGHT", 10, 0);
	setHintsInput:SetScript("OnEnterPressed", setHintsInput_OnEnterPressed);
	setHintsInput:SetScript("OnEscapePressed", setHintsInput_OnEscapePressed);
	setHintsInput:SetScript("OnEnter",setControlTooltip);
	setHintsInput:SetScript("OnLeave",GameTooltip_Hide);

	-- hints label
	local setHintsInputlabel = setHintsInput:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	setHintsInputlabel:SetText(L["Hints"]);
	setHintsInputlabel:SetPoint("BOTTOMLEFT", setHintsInput, "TOPLEFT", -4, -2);	

	-- Next record
	local nextbtn = CreateFrame("Button", "TriviaBot_QuestionMakerNextBtn", gui, "OptionsButtonTemplate");
	gui.nextRecordbtn = nextbtn
	nextbtn:SetWidth(100);
	nextbtn:SetText(L["Next"]);
	nextbtn:SetPoint("BOTTOMRIGHT", questionSection, "BOTTOMRIGHT", -2, 5);
	nextbtn:SetScript("OnClick", nextbtn_OnClick);
	nextbtn:SetScript("OnEnter",setControlTooltip);
	nextbtn:SetScript("OnLeave",GameTooltip_Hide);
	nextbtn:SetFrameLevel(questionSection:GetFrameLevel()+1);
	
	-- Test active set on TriviaBot if loaded
	if IsAddOnLoaded("TriviaBot") then
		local testbtn = CreateFrame("Button", "TriviaBot_QuestionMakerTriviaBotBtn", gui, "OptionsButtonTemplate");
		gui.testSetbtn = testbtn
		testbtn:SetWidth(100);
		testbtn:SetText(L["Test"]);
		testbtn:SetPoint("BOTTOMLEFT", questionSection, "BOTTOMLEFT", 20, 5);
		testbtn:SetScript("OnClick", testbtn_OnClick);
		testbtn:SetScript("OnEnter",setControlTooltip);
		testbtn:SetScript("OnLeave",GameTooltip_Hide);
		testbtn:SetFrameLevel(questionSection:GetFrameLevel()+1);
		testbtn:Disable();
	end
	
	-- Save (reloads)
	local savebtn = CreateFrame("Button", "TriviaBot_QuestionMakerSaveBtn", gui, "OptionsButtonTemplate");
	gui.reloadbtn = savebtn
	savebtn:SetWidth(100);
	savebtn:SetText(L["SaveSet"]);
	savebtn:SetPoint("BOTTOM", gui, "BOTTOM", 0, 10);
	savebtn:SetScript("OnClick", savebtn_OnClick);
	savebtn:SetScript("OnEnter",setControlTooltip);
	savebtn:SetScript("OnLeave",GameTooltip_Hide);
	savebtn:Disable();
	savebtn:SetMotionScriptsWhileDisabled(true)
	
	gui:Hide()
	return gui
	
end

-- script handlers
function f.ADDON_LOADED(...)
	local addon = ...
	if addon == "TriviaBot_QuestionMaker" then
		TriviaBot_QuestionsQM = TriviaBot_QuestionsQM or {}
		f.gui = createGUI()
		SlashCmdList["TRIVIABOTQM"] = 
		function(msg)
			if not f.gui:IsShown() then
				f.gui:Show()
			end 
		end
		SLASH_TRIVIABOTQM1 = "/tbqm"
		if next(TriviaBot_QuestionsQM) then
 			TB_Question_Sets = TriviaBot_QuestionsQM
 		end
	end
end

-- exported to global namespace for TriviaBot interoperation and keybinds
local Exports = {}
Exports["nextRecord"] = function() if f.gui:IsShown() then nextbtn_OnClick(f.gui.nextRecordbtn) end end
Exports["refreshData"] = NOP -- replace this with an actual function to pull data TriviaBot puts in its own Exports.
Exports["ActiveSet"] = {}
_G["TriviaBotQuestionMakerExports"] = Exports
